<?php
namespace Controller;

use Service\Traits\TResponse;
use Service\ConnectionService;
use Service\DeleteCascadeService;
use Util\Environment;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\Manager;
use Util\Utilities;

class ApiUsageLog extends Crud  {

    use TResponse;

    protected $dataSourceName = 'apiUsageLog';
    /** @var \Service\Manager\Mongo\Manager  */
    protected $organizationMongoManager;
    /** @var Environment */
    protected $environment;

    public function __construct(\Service\Manager\Mongo\Manager $organizationMongoManager, DeleteCascadeService $deleteCascadeService, Environment $environment) {
        parent::__construct($organizationMongoManager, $deleteCascadeService);
        $this->organizationMongoManager = $organizationMongoManager;
        $this->environment = $environment;
    }

    public function findNext(DTO\Request $request, DTO\Response $response)
    {
        $params = $request->getParameters();
        /*
            if(empty($sort)){
                $sort = array("CREATED_AT" => -1 );
            }
        */
        $data = $this->mongoManager->findNext($this->dataSourceName, $params);
        $data = Utilities::parseDateGrid($data, ["CREATED_AT"]);
        $data = Utilities::parseDataSourceZeedhiNext($data);

        // Paginação
        $data["pagination"]["total"] = $this->count($params);
        $data["pagination"]["page"] = array_key_exists("page", $params) ? intval($params["page"]) : 1;
        $data["pagination"]["limit"] = array_key_exists("limit", $params) ? intval($params["limit"]) : 9999;
        $this->sendResponse($data, 200);
        // Utilities::datasetMongoToZeedhi($response, $dataSet,$this->dataSourceName);

    }

    public function count($params) {
	    return $this->mongoManager->countNext($this->dataSourceName, $params);
	}
}