<?php

namespace Service\ChartView;

use Zeedhi\Framework\DependencyInjection\InstanceManager;
class Grid extends ChartViewAbstract implements ChartViewInterface {

    const TYPE_ID_GRID = "56200912f038efbfa1f75105";

    public $widget;
    public $widgetData;

    function __construct($widget = null) {
        $this->widget = $widget;
        $this->widgetData = [];
    }

    public function getColumns(){
        $columns = [];
        foreach(array_merge($this->widget["parameters"]["dimensions"], $this->widget["parameters"]["facts"]) as $zmartColumn){
            $columnParameters = $this->widget["fields"][array_search($zmartColumn, array_column($this->widget["fields"], "name"))];
            $column = [
                "name" => $zmartColumn,
                "label" => !empty($columnParameters["label"]) ? $columnParameters["label"] : $zmartColumn 
            ];
            if(!empty($columnParameters["type"]) && $columnParameters["type"] == "fact"){
                $column["aggregation"] = $columnParameters["expression"];
            }
            $columns[] = $column;
        }
        if(!empty($this->widget["zmartTotalized"])){
            $groupedField = $this->getGroupedField($this->widget["fields"]);
            if(!empty($groupedField)){
                $columns[] = [
                    "name" => $groupedField["name"],
                    "grouped" => true
                ];
            }
        }
        return $columns;
    }

    protected function getGroupedField($fields){
        return $fields[array_search(true, array_map(function($field){return !empty($field["grouped"]);}, $fields))];
    }

    public function getDatasource(){
        $datasource = [
            "data" => $this->widgetData,
            "type" => "tek-memory",
            "limit" => 25
        ];
        return $datasource;
    }

    public function convertToNext():array {
        $this->fetchWidgetData();
        $widget = $this->widget;
        $indicatorNext = [
            "name" => "apexChart_" . $widget['id'],
            "id" => $widget['id'],
            "widget" => $widget,
            "viewTypeId" => self::TYPE_ID_GRID,
            "component" => "TekGrid",
            "title" => !empty($widget["label"]) ? $widget["label"] : "",
            "showExport" => true,
            "showReload" => false,
            "height" =>"100%",
            "cssStyle" => self::CHART_CSS_STYLE,
            "columnsButton" => true,
            "columns" => $this->getColumns(),
            "datasource" => $this->getDatasource(),
            // "events" => [
            //     "rowClick" => "{{ChartViewController.drillGrid}}",
            // ]
        ];

        $this->widget = $indicatorNext;
        return $indicatorNext;
    }
}
