import { Metadata, Http, DateHelper } from '@zeedhi/core';
import { Tab, Tabs, Form, Modal, ModalService, Select, DateRange, Dashboard, Text } from '@zeedhi/common';
import { PeriodsService } from '@/Services/PeriodsService';
import { SkeletonMetadata } from '@/Services/SkeletonMetadata';
import { ZmartChartsApex } from '../Services/Chats/ZmartChartsApex';
import { ChartViewFactory } from '../Services/ChartView/ChartViewFactory';
import { Ordination } from '../Services/Ordination/Ordination';
import { Filter } from '../Services/Filter/Filter';

export class DashboardController {
    public tabsHeader: any = '';
    public dashboards: any = '';
    public widget: any = '';
    public countNameFormDashboard: any = 0;
    public nameFormDashboard: any = '';
    public headerTabs: any[] = [];
    public viewTypeId: any;
    public nameTypeChart: any;
    public dashboardCards: any;
    public dashboardId: any;
    public tabIndex: any = '';
    public countSkeleton: any = 0;
    public filterReloadDashboard: any = '';
    public getLastImportationDate: any = '';
    private filterDashboardModal!: Modal;
    periodFilterDashboard: any[] = [];
    private structureLabelFilterDashboard: any = null;
    periodFilter: any[] = [];
    public modalDrilldown!: Modal;
    public modalHiddenIndicators: any = undefined;
    public cardsListBeforeEdit: any = {};
    public hiddenCards: any = [];
    public forceDashboardSave: boolean = false;

    public async openDrillModal() {
        ZmartChartsApex.isDashboard = false;
        this.modalDrilldown?.destroy();
        this.modalDrilldown = await ModalService.createFromJson("chartView", true);

        const modalName = this.modalDrilldown.name + "_drill_" + Date.now().toString();
        this.modalDrilldown.name = modalName;
        this.modalDrilldown.fullscreen = true;
        this.modalDrilldown.children.unshift({
                "name": "fullscreenHeader",
                "component": "ZdHeader",
                "color": "transparent",
                "padless": true,
                "elevation": 0,
                "cssClass": "zd-mb-4",
                "rightSlot": [
                  {
                    "name": modalName + "_closeModalButton",
                    "component": "ZdButton",
                    "small": true,
                    "icon": true,
                    "iconName": "close",
                    "events": {
                        "click": this.closeDrillModal.bind(this, this.modalDrilldown)
                    }
                  }
                ]
        });
        this.modalDrilldown.show();
    }

    public closeDrillModal(modal: Modal) {
        modal.hide();
        localStorage.setItem('drillDownView', "");
        localStorage.setItem('drillDownFilter', "");
        ZmartChartsApex.isDashboard = true;
    }

    public async loadTabs() {
        this.filterReloadDashboard = '';
        const tab = Metadata.getInstance<Tabs>('tabsDashboard');
        tab.tabs = [];
        const buildMainDashboards = await Filter.filterLoadTabs();
        this.tabsHeader = buildMainDashboards.data.dataset.dashboards;

        this.tabsHeader.forEach((tabs: any) => {
            const id = tabs._id;
            const dashName = "dashTab_" + id;
            const dashHeight = (window.innerHeight - 168) + "px";
            const tabb = new Tab(
                {
                    "name": "tab_" + id,
                    "tabTitle": tabs.name,
                    "teste": false,
                    "scrollView": false,
                    "children": [
                        {
                            "name": "tabDashboard_" + id,
                            "component": "ZdContainer",
                            "cssClass": "zd-pa-0",
                            "scrollView": true,
                            "maxHeight": dashHeight,
                            "children": [
                                {
                                    "name": dashName,
                                    "id": id,
                                    "zmartSelected": false,
                                    "zmartSelectedFilter": false,
                                    "component": "ZdDashboardGrid",
                                    "cssStyle": "padding: 0;",
                                    "editingMode": true,
                                    "showGridLines": false,
                                    "removePadding": true,
                                    "isVisible": false,
                                    "colNum": 18,
                                    "margin": [10, 10],
                                    "cards": [],
                                    "events": {
                                        "layoutUpdated": this.dashboardLayoutUpdated.bind(this),
                                    },
                                    "showFullscreen": "top",
                                    "showEditHeader": false,
                                    "editHeader": {
                                        "name": dashName + "_invisibleHeader",
                                        "cssStyle": "display: none !important;",
                                    },
                                    "cardFooterSlot": [
                                        {
                                          "name": dashName + "_cardFooter",
                                          "component": "ZdTooltip",
                                          "bottom": true,
                                          "label": "HIDE",
                                          "children": [
                                            {
                                              "name": dashName + "_hideBtn",
                                              "component": "ZdButton",
                                              "icon": true,
                                              "iconName": "mdi-eye-off",
                                              "color": "info",
                                              "events": {
                                                "click": this.hideCard.bind(this, dashName),
                                              }
                                            },
                                          ]
                                        }
                                      ],
                                }
                            ]
                        },
                        SkeletonMetadata.skeleton(id)
                    ]
                }
            );
            tab.tabs.push(tabb);
        });
        tab.activeTab = 0;
        this.tabIndex = tab.activeTab;
         if (tab.activeTab === 0 && this.tabsHeader[0]) {

            this.dashboards = await Filter.dashboardFilter(this.tabsHeader[0].name);

            this.getTabComponent(tab.tabs[this.tabIndex]).zmartSelected = true; // ZMARTSELECTED como true caso a TAB já tenha sido clicada (Visitada).
            let dashboardIndicators = this.dashboards.data.dataset.mainDashboard.indicators;
            if (!dashboardIndicators) {
                dashboardIndicators = this.dashboards.data.dataset.mainDashboard[0].indicators;
            }
            this.dashboardsTabs(dashboardIndicators, 0);
        }
    }

    private dashboardIsChanged(event: any) {
        if (Object.keys(this.cardsListBeforeEdit).length === 0) {
            return false;
        }
        return event.some((card: any) => this.cardsListBeforeEdit[card.i] && (this.cardsListBeforeEdit[card.i].w !== card.w || this.cardsListBeforeEdit[card.i].h !== card.h || this.cardsListBeforeEdit[card.i].x !== card.x || this.cardsListBeforeEdit[card.i].y !== card.y));
    }

    public async dashboardLayoutSave(event: any) {
        if (this.dashboardIsChanged(event) || this.forceDashboardSave) {
            this.forceDashboardSave = false;
            event = event.map(function (card:any) {
                if (card.hidden) {
                    delete card.hidden;
                }
                return card;
            });
            if (this.hiddenCards) {
                const hiddenCardsList = this.hiddenCards.map(function (card:any) {
                    const newCard = card;
                    card.hidden = true;
                    return newCard;
                });
                event = [...event, ...hiddenCardsList];
            }
            const data = {
                'dataset': [
                    {
                        "viewId": this.tabsHeader[this.tabIndex]._id,
                        "data": event,
                    }
                ],
                "disableLoader": false,
                "requestType": 'DataSet'
            };
            try {
                this.dashboardSetSavingText(true);
                await Http.post('/view/saveUserView', data);
                this.dashboardSetSavingText(false);
            } catch (err) {
                console.error(err);
            } finally {
                this.dashboardSetSavingText(false);
            }
        }
    }

    public dashboardLayoutUpdated({ event }: any) {
        this.dashboardLayoutSave(event);
    }

    private dashboardSetSavingText(saving: boolean) {
        const text = Metadata.getInstance<Text>("dashboardSavingText");
        text.isVisible = saving;
    }

    public async closeModalVisibilityCard() {
        this.modalHiddenIndicators.hide();
    }

    public async openModalVisibilityCard() {
        if (!this.modalHiddenIndicators) {
            this.modalHiddenIndicators = this.createIndicatorVisibilityModal();
        }
        this.reloadHiddenIndicatorsDatasource();
        this.modalHiddenIndicators.show();
    }

    public async showHiddenCard({ row, component }: any) {
        component.disabled = true;
        const index = this.hiddenCards.findIndex((obj: { i: any }) => obj.i === row.i);
        if (index !== -1) {
            this.hiddenCards.splice(index, 1);
            const tabName = Metadata.getInstance<Tabs>("tabsDashboard");
            this.getTabComponent(tabName.tabs[this.tabIndex]).cards.push(row);
            this.reloadHiddenIndicatorsDatasource();
            this.forceDashboardSave = true;
        }
    }

    public async hideCard(dashName: any) {
        const dashboard = Metadata.getInstance<Dashboard>(dashName);
        const tabName = Metadata.getInstance<Tabs>("tabsDashboard");
        const index = this.getTabComponent(tabName.tabs[this.tabIndex]).cards.findIndex((obj: { i: any }) => obj.i === dashboard.currentCardId);
        if (index !== -1) {
            const removedCard = this.getTabComponent(tabName.tabs[this.tabIndex]).cards.splice(index, 1);
            this.hiddenCards.push(removedCard[0]);
        }
    }

    private reloadHiddenIndicatorsDatasource() {
        this.modalHiddenIndicators.children[0].datasource.data = this.hiddenCards;
        if (this.modalHiddenIndicators.childrenInstances && this.modalHiddenIndicators.childrenInstances.length && this.modalHiddenIndicators.childrenInstances.length > 0) {
            if (this.modalHiddenIndicators.childrenInstances.length > 2) {
                this.modalHiddenIndicators.childrenInstances = [
                    this.modalHiddenIndicators.childrenInstances[2],
                    this.modalHiddenIndicators.childrenInstances[3]
                ];
            }
            this.modalHiddenIndicators.childrenInstances[0].reload();
        }
    }

    private createIndicatorVisibilityModal() {
        return ModalService.create({
            name: 'visibilityModal_',
            persistent: true,
            title: 'VISIBILITY_MODAL',
            children: [
                {
                    name: 'listCardVisibility_',
                    component: 'TekGrid',
                    cssClass: 'zd-my-4',
                    showHeader: false,
                    showFooter: false,
                    showLayoutOptions: false,
                    showReload: false,
                    showSearch: true,
                    showSelectAllGrid: false,
                    showSummaryTotal: false,
                    toolbarSlot: [],
                    columns: [
                        {
                            name: 'name',
                            label: 'Name',
                            sortable: false,
                        },
                        {
                            name: 'toggleVisibilityAction_',
                            type: 'action',
                            children: [
                                {
                                    name: 'toggleVisibilityBtn_',
                                    component: 'ZdButton',
                                    disabled: false,
                                    icon: true,
                                    small: true,
                                    color: 'primary',
                                    iconName: 'eye',
                                    events: {
                                        click: this.showHiddenCard.bind(this),
                                    },
                                },
                            ],
                        },
                    ],
                    datasource: {
                        uniqueKey: 'i',
                        data: this.hiddenCards,
                    },
                },
                {
                    name: 'visibilityCardFooter_',
                    component: 'ZdFooter',
                    color: 'transparent',
                    rightSlot: [
                        {
                            name: 'visibilityCancelButton_',
                            component: 'ZdButton',
                            cssClass: 'zd-mr-2',
                            outline: true,
                            label: 'CLOSE',
                            events: {
                                click: this.closeModalVisibilityCard.bind(this),
                            },
                        },
                    ],
                },
            ],
        });
    }

    private getTabComponent(tabs:any) {
        return tabs.children[0].children[0];
    }

    public async clickTab({ tabIndex }: any) {
        this.tabIndex = tabIndex;
        const tab = Metadata.getInstance<Tabs>('tabsDashboard');
        if (this.getTabComponent(tab.tabs[tabIndex]).zmartSelected === false) {

            this.dashboards = "";
            this.dashboards = await Filter.dashboardFilter(this.tabsHeader[tabIndex].name);

            if (this.periodFilterDashboard[0] !== undefined) {
                this.getTabComponent(tab.tabs[tabIndex]).cards = [];
                this.dashboardsTabs(this.dashboards.data.dataset.mainDashboard.indicators, tabIndex, true);
            } else {
                this.dashboardsTabs(this.dashboards.data.dataset.mainDashboard.indicators, tabIndex);
            }
        }
        this.getTabComponent(tab.tabs[tabIndex]).zmartSelected = true;
        this.getTabComponent(tab.tabs[tabIndex]).zmartSelectedFilter = true;
    }

    public async dashboardsTabs(dashboards: any, tabIndex: any, filter: boolean = false) {
        this.hiddenCards = [];
        const tab = Metadata.getInstance<Tabs>('tabsDashboard');
        const tabComponent = this.getTabComponent(tab.tabs[this.tabIndex]);
        if (tabComponent.zmartSelectedFilter === false) {
            const indicators: any[] = [];
            const dashboardIndicatorsIds: any[] = [];
            const tabName = Metadata.getInstance<Tabs>("tabsDashboard");
            this.getTabComponent(tabName.tabs[tabIndex]).cards = [];

            if (this.filterReloadDashboard !== '') {
                dashboards.forEach(async (dashboard: any) => {
                    dashboard.parameters.period.start = this.periodFilterDashboard[0];
                    dashboard.parameters.period.end = this.periodFilterDashboard[1];
                });
            }

            dashboards.forEach(function (dashboard: any) {
                if (!dashboardIndicatorsIds.includes(dashboard.indicator)) {
                    dashboardIndicatorsIds.push(dashboard.indicator);
                }
            });

            if (dashboards[0] !== undefined) {
                this.getLastImportationDate = "";
                this.getLastImportationDate = await Filter.filterGetLastImportationDates(dashboardIndicatorsIds, this.tabsHeader[tabIndex].name);
            }
            let userViews = await Http.get("/view/getUserViews");
            if (userViews && userViews.data && userViews.data[tabComponent.id]) {
                userViews = userViews.data[tabComponent.id];
            } else {
                userViews = null;
            }

            Ordination.sort(dashboards); // Ordenando os indicadores
            let widthCounter = 0;
            let heightCounter = 0;
            let biggestHeight = 0;
            const loadDashboards = async () => {
                this.viewTypeId = {};
                this.dashboardCards = {};
                let addToIndicator = true;
            dashboards.forEach(async (dashboard: any) => {
                addToIndicator = true;
                if (widthCounter + dashboard.width > 18) {
                    heightCounter += biggestHeight;
                    widthCounter = 0;
                    biggestHeight = 0;
                }
                this.dashboardCards["override_" + dashboard._id] = dashboard;
                if (!dashboard.width) {
                    dashboard.width = 3;
                }
                if (!dashboard.height) {
                    dashboard.height = 3;
                }
                const indicator = {
                    "i": dashboard._id,
                    "path": "dashboard/dashboards",
                    "local": true,
                    "overrideNamedProps": {
                        "NAME": "override_" + dashboard._id,
                    },
                    "children": [],
                    "fillHeight": true,
                    "cssStyle": "height: 100% !important;",
                    "h": dashboard.height ? dashboard.height : 3,
                    "w": dashboard.width ? dashboard.width : 3,
                    "x": (dashboard.x === undefined) ? widthCounter : dashboard.x,
                    "y": (dashboard.y === undefined) ? heightCounter : dashboard.y,
                };
                if (userViews) {
                    if (userViews[dashboard._id]) {
                        indicator.h = userViews[dashboard._id].h;
                        indicator.w = userViews[dashboard._id].w;
                        indicator.x = userViews[dashboard._id].x;
                        indicator.y = userViews[dashboard._id].y;
                    } else if (userViews.hidden[dashboard._id]) {
                        indicator.h = userViews.hidden[dashboard._id].h;
                        indicator.w = userViews.hidden[dashboard._id].w;
                        indicator.x = userViews.hidden[dashboard._id].x;
                        indicator.y = userViews.hidden[dashboard._id].y;
                        addToIndicator = false;
                    }
                }
                if (addToIndicator) {
                    indicators.push(indicator);
                    widthCounter += dashboard.width;
                    if (biggestHeight === 0 || biggestHeight < dashboard.height) {
                        biggestHeight = dashboard.height;
                    }
                } else {
                    this.hiddenCards.push(indicator);
                }
            });
                this.cardsListBeforeEdit = this.getCardsListById(indicators);
                this.getTabComponent(tabName.tabs[tabIndex]).cards = indicators;
            };
            loadDashboards();
        }

        if (filter === true) {
            this.getTabComponent(tab.tabs[tabIndex]).zmartSelected = true;
            this.getTabComponent(tab.tabs[tabIndex]).zmartSelectedFilter = true;
        }

    }

    private getCardsListById(indicators: any) {
        return indicators.reduce((obj: any, item: any) => {
            obj[item.i] = {
                "h": item.h,
                "w": item.w,
                "x": item.x,
                "y": item.y,
            };
            return obj;
          }, {});
    }

    public async onMountedChildrenDashboard({component}: any) {
        const dashboard = this.dashboardCards[component.name];
        const tabName = Metadata.getInstance<Tabs>("tabsDashboard");
        let period = "";
        if (dashboard.parameters && dashboard.parameters.period) {
            period = dashboard.parameters.period;
        }
        let structureId = "";
        if (this.structureLabelFilterDashboard) {
            structureId = this.structureLabelFilterDashboard;
        }
        const buildWidget = await Filter.filterBuildWidgetByViewIdNext(period, dashboard._id, structureId);
        this.widget = '';
        this.widget = buildWidget.data.dataset.widget;
        this.viewTypeId[this.widget[0].id] = dashboard.viewTypeId;
        await ChartViewFactory.getChartType(this.viewTypeId[this.widget[0].id], dashboard, this.widget);
        const charts = ZmartChartsApex.build(this.widget, this, component);
        const widgetChats = charts.getCharts();
        widgetChats[0].grid = {
            "sm": "12",
            "lg": "12",
            "cols": "18",
            "md": "12",
        };
        widgetChats[0].height = this.heightCard(dashboard.height - 1) + "px";
        const componentDataImportation = this.periodLabel(dashboard);

        widgetChats.push(componentDataImportation); // Setando na última posição do Json
        if (this.widget[0].name === 'notFound') {
            component.children = this.widget;
            this.getTabComponent(tabName.tabs[this.tabIndex]).isVisible = true;
            tabName.tabs[this.tabIndex].children[1].isVisible = false;
        } else {
            component.children = widgetChats;
            this.getTabComponent(tabName.tabs[this.tabIndex]).isVisible = true;
            tabName.tabs[this.tabIndex].children[1].isVisible = false;
        }
    }

    public periodLabel(dashboard : any) {
        let componentDataImportation: any = {};
        const updateDate = this.getLastImportationDate.data.dataset.lastImportDates[dashboard.dataSourceId];
        if (dashboard.parameters.period) {
            let componentText: string = "";

            if (dashboard.parameters.period.periodLabel === undefined) {
                componentText = dashboard.parameters.period.start + " - " + dashboard.parameters.period.end;
            } else {
                componentText = dashboard.parameters.period.periodLabel;
            }

            if (updateDate) {
                componentText += " * Atualizado em : " + updateDate;
            }
            const dashId = dashboard.id;
            componentDataImportation = {
                "name": dashId + "_callDescriptionValue_container",
                "component": "ZdContainer",
                "width": "100%",
                "grid": {
                    "sm": "12",
                    "lg": "12",
                    "cols": "18",
                    "md": "12",
                },
                "children": [
                    {
                        "name": dashId + "_callDescriptionValue",
                        "component": "ZdText",
                        "width": "100%",
                        "cssClass": "zd-text-right",
                        "cssStyle": "font-size: 11px; color: #aaa; line-height: 15px; font: Courier, monospace;",
                        "text": componentText,
                    }
                ]
            };
        }
        return componentDataImportation;
    }

    public heightCard(height: any) {
        return height * 130;
    }

    public instanceTab() {
        const tabs = Metadata.getInstance<Tabs>('tabsDashboard');

        tabs.tabs.forEach(function (tab: any) {
            tab.children[0].children[0].zmartSelectedFilter = false;
            tab.children[0].children[0].zmartSelected = false;
            tab.children[0].children[0].isVisible = false;
            tab.children[1].isVisible = true;
        });
    }

    public async reload() {
        this.instanceTab();
        const dashboards = await Filter.dashboardFilter(this.tabsHeader[this.tabIndex].name);
        const dashboardsIndicators = dashboards.data.dataset.mainDashboard.indicators;
        this.filterReloadDashboard = '';
        if (this.periodFilterDashboard[0] !== undefined) {
            this.filterReloadDashboard = dashboardsIndicators;
        }

        if (this.periodFilterDashboard[0] !== undefined) {
            dashboardsIndicators.forEach((dashboard: any) => {
                dashboard.parameters.period = {
                    "start": this.periodFilterDashboard[0],
                    "end": this.periodFilterDashboard[1],
                };
            });
            this.filterReloadDashboard = dashboardsIndicators;
            this.dashboardsTabs(dashboardsIndicators, this.tabIndex, true);
        } else {
            this.dashboardsTabs(dashboardsIndicators, this.tabIndex);
        }
    }

    public filterDashboard() {
        this.createFilterDashboardModal();
    }

    public async createFilterDashboardModal() {
        this.filterDashboardModal = await ModalService.createFromJson('dashboard/modal/filterDashboard', true);
        this.filterDashboardModal.show();
    }

    public closeFiterDashboardModal() {
        this.filterDashboardModal.hide();
        this.filterDashboardModal?.destroy();
    }

    public async onMountedFilterDashboard() {
        // Se formulário não foi montado, não faz nada
        if (Metadata.getInstances('structureTypeForm').length === 0 || !Metadata.getInstances('structureTypeForm')[0]) {
            return;
        }
        const form = Metadata.getInstance<Form>('structureTypeForm');
        const structureLabel = Metadata.getInstance<Select>('structure_label');
        const period = form.getChildInstance<DateRange>('period');

        const periodDropdown = period.childrenInstances[0].childrenInstances[0];
        periodDropdown.maxHeight = 190;

        this.generatePeriodHelperOptions(period);
        const data = await Filter.filterGetStructureByUser();

        structureLabel.datasource.updateData(data.data.dataset.getStructureByUser);

        if (this.periodFilterDashboard[0] !== undefined) {
            form.value.period = this.periodFilter;
        }
    }

    public async generatePeriodHelperOptions(period: any) {
        const periodDate = await Http.post("/periods");
        const data = periodDate.data.data;
        if (period.helperOptions && period.helperOptions.length > 0) {
            period.helperOptions.forEach((option: any) => DateHelper.unregister(option.periodsCode));
        }
        data.forEach((option: any) => DateHelper.register(option.periodsCode, option.name, () => {
            const service = new PeriodsService();
            const periods = service.decodePeriod(option.periodsCode);
            return [periods.start, periods.end].map((date: any) => date.split('/').reverse().join('-')) as any;
        }));
        period.helperOptions = data.map((option: any) => option.periodsCode).filter((val: any) => val);
        period.helperValue = data.periodsCode;
        period.updateHelperHint();
    }

    public async saveFiterDashboard() {
        const form = Metadata.getInstance<Form>('structureTypeForm');
        let dataForm = false;
        if (!!form.value.period && !!form.value.period.length) {
            const dateStart = form.value.period[0];
            const dateEnd = form.value.period[1];
            dataForm = true;
            this.periodFilterDashboard = [dateStart, dateEnd];
            this.periodFilter = [dateStart, dateEnd];
        }
        if (dataForm === false) {
            this.periodFilterDashboard = [];
        }

        if (form.value.structure_label !== undefined) {
            this.structureLabelFilterDashboard = form.value.structure_label;
        }

        this.closeFiterDashboardModal();
        await this.reload();
    }
}
