import { Metadata, FormatterParserProvider, Http, Validation } from '@zeedhi/core';
import { Modal, ModalService, Grid, Form, LoadingService, AlertService, DialogService } from '@zeedhi/common';
import { CustomAlertService } from '@/Services/CustomAlertService';

//  Formata texto da coluna Periodo
FormatterParserProvider.registerFormatter('PeriodoFormatter', (periodObject: any) => "De " + periodObject.start + " até " + periodObject.end);
//  Funciona como um ZdSelectMultiple, iterando pelos valores e chamando ZdSelect
FormatterParserProvider.registerFormatter('column_SelectMultipleFormatter', (params: any) => {
    const formatterFn = FormatterParserProvider.getFormatter('column_ZdSelect');
    const oldValue = params.value;
    if (params.componentProps.datasource.data) {
        params.column.loading = false;
    }
    if (!oldValue?.length) {
        return "";
    }
    const formatedValue = oldValue.map((value: any) => formatterFn({...params, value: [value]})).map((val: any) => (val instanceof Array ? val[0] : val));
    if (formatedValue.every((v: any, i: any) => v === oldValue[i])) {
        return "";
    }
    return formatedValue.join(', ');
});
//  Validação do Date Range
const newValidation = (config: any) => (value: string[]) => value.length !== 1 || config.message || "Duas datas devem ser apresentadas";
Validation.register('dateRangeValidation', newValidation);
//  Controller
export class PeriodObservationController {
    //  Metodos de dialog
    public dialogEvents: any = {
        click: async ({ component }: any) => {
            if (component.name === "sim") {
                await this.deleteRow();
            }
            DialogService.hide();
        }
    };
    public showDeleteDialog({ event }: any) {
        event.preventDefault();
        DialogService.show({
          name: 'delete-dialog',
          type: 'warning',
          title: ' ',
          text: 'Deseja remover o(s) registro(s) selecionado(s)?',
          buttons: [
            { name: 'não', component: 'ZdButton', label: 'NÃO', events: this.dialogEvents, outline: true },
            { name: 'sim', component: 'ZdButton', label: 'SIM', events: this.dialogEvents },
          ],
        });
    }
    //  Metodos de Modal
    private detailModal!: Modal;
    public async createDetailModal() {
        this.detailModal = await ModalService.createFromJson('periodObservation/periodDetailModal', true);
    }
    public destroyDetailModal() {
        this.detailModal?.destroy();
    }
    public closeDetailModal() {
        this.detailModal.hide();
    }
    //  Metodos do Formulario
    protected modalData: any;
    public get currentRowId() {
        return this.modalData._id;
    }
    public get modalTitle() {
        return (this.currentRowId ? "Editar" : "Adicionar") + " Observação de Período";
    }
    public formOnMounted({ component }: any) {
        const form = component;
        const data = this.modalData;
        form.getChildInstance('name').value = data.name;
        form.getChildInstance('period').setDateValue(data.period.start + ' - ' + data.period.end);
        form.getChildInstance('indicator').setValue(data.indicator);
        form.getChildInstance('dataSource').setValue(data.dataSource);
    }
    public async openDetailModal({ component, row }: any) {
        if (component.editing === true) return;
        this.destroyDetailModal();
        await this.createDetailModal();
        const defaultValues = {name: "", period: "", indicator: [], dataSource: []};
        this.modalData = row === undefined ? defaultValues : row;
        this.detailModal.show();
    }
    public saveDetailModal() {
        debugger;
        const form = this.detailModal.getChildInstance<Form>("detailForm");
        const values = form.value;
        if (!form.validate()) {
            return;
        }
        const content = {
            name: values.name,
            period: {start: values.period[0], end: values.period[1]},
            indicator: values.indicator?.length > 0 ? values.indicator : null,
            dataSource: values.dataSource?.length > 0 ? values.dataSource : null,
            _id: this.currentRowId,
            "changed": true
        };
        const addContent = {
            __createdLocal: true,
            __is_new: true,
            ...content
        };
        Http.post("periodObservation/save", {
            "requestType": "DataSet",
            "disableLoader": false,
            "dataset": [this.currentRowId ? content : addContent]
        }).then(() => {
            CustomAlertService.saveSuccessAlert();
        }).catch(() => {
            CustomAlertService.errorAlert("Ocorreu um erro.");
        }).finally(() => {
            this.reload();
            this.closeDetailModal();
        });
    }
    //  Grid Methods
    public gridOnMounted() {
        const botaoRefreshGrid: any = Metadata.getInstance("period-observation-grid_refreshButton");
        botaoRefreshGrid.events.click = this.reload.bind(this);
        this.reload();
    }
    public deleteRow() {
        const grid = Metadata.getInstance<Grid>('period-observation-grid');
        return Http.post("/periodObservation/delete", {
            "requestType": "Row",
            "row": grid.selectedRows
        }).then(() => {
            CustomAlertService.deleteSuccessAlert();
        }).catch(() => {
            CustomAlertService.errorAlert("Ocorreu um erro.");
        }).finally(() => {
            this.reload();
            grid.selectedRows = [];
        });
    }
    //  public isDataLoading = true;
    public indicAc = [];
    public async reload() {
        //  Os códigos a seguir tem unicamente o proposito de apresentar Loading enquanto os dados de
        //  this.isDataLoading = true;
        const grid = Metadata.getInstance<Grid>('period-observation-grid');
        const indicatorRequest = Http.get("indicator");
        const dataSourceRequest = Http.get("dataSource");
        grid.getColumn("indicator").loading = true;
        grid.getColumn("dataSource").loading = true;
        return Promise.all([indicatorRequest, dataSourceRequest, grid.datasource.get()]).then(([indicatorResponse, dataSourceResponse]) => {
            grid.getColumn("indicator").componentProps.datasource.data = indicatorResponse.data.data;
            grid.getColumn("dataSource").componentProps.datasource.data = dataSourceResponse.data.data;
        }).catch(console.error);
    }
    public editPeriodInline({ component }: any) {
        const rows = component.getEditedRows();
        rows.forEach((val: any) => { val.changed = true; });
         Http.post("periodObservation/save", {
            "requestType": "DataSet",
            "disableLoader": false,
            "dataset": rows
        }).then(() => {
            CustomAlertService.saveSuccessAlert();
        }).catch(() => {
            CustomAlertService.errorAlert("Ocorreu um erro.");
        }).finally(() => {
            this.reload();
        });
    }
}
